@@ -1,16 +1,14 @@
 # postgresql/psycopg2.py
-# Copyright (C) 2005-2016 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
-
-"""
+r"""
 .. dialect:: postgresql+psycopg2
     :name: psycopg2
     :dbapi: psycopg2
-    :connectstring: postgresql+psycopg2://user:password@host:port/dbname\
-[?key=value&key=value...]
+    :connectstring: postgresql+psycopg2://user:password@host:port/dbname[?key=value&key=value...]
     :url: http://pypi.python.org/pypi/psycopg2/
 
 psycopg2 Connect Arguments
@@ -31,6 +29,7 @@ psycopg2-specific keyword arguments which are accepted by
   Note that the :paramref:`.Connection.execution_options.stream_results`
   execution option is a more targeted
   way of enabling this mode on a per-execution basis.
+
 * ``use_native_unicode``: Enable the usage of Psycopg2 "native unicode" mode
   per connection.  True by default.
 
@@ -53,6 +52,16 @@ psycopg2-specific keyword arguments which are accepted by
 
     :ref:`psycopg2_unicode`
 
+* ``use_batch_mode``: This flag allows ``psycopg2.extras.execute_batch``
+  for ``cursor.executemany()`` calls performed by the :class:`.Engine`.
+  It is currently experimental but
+  may well become True by default as it is critical for executemany
+  performance.
+
+  .. seealso::
+
+    :ref:`psycopg2_batch_mode`
+
 Unix Domain Connections
 ------------------------
 
@@ -67,13 +76,12 @@ in ``/tmp``, or whatever socket directory was specified when PostgreSQL
 was built.  This value can be overridden by passing a pathname to psycopg2,
 using ``host`` as an additional keyword argument::
 
-    create_engine("postgresql+psycopg2://user:password@/dbname?\
-host=/var/lib/postgresql")
+    create_engine("postgresql+psycopg2://user:password@/dbname?host=/var/lib/postgresql")
 
-See also:
+.. seealso::
 
-`PQconnectdbParams <http://www.postgresql.org/docs/9.1/static/\
-libpq-connect.html#LIBPQ-PQCONNECTDBPARAMS>`_
+    `PQconnectdbParams \
+    <http://www.postgresql.org/docs/9.1/static/libpq-connect.html#LIBPQ-PQCONNECTDBPARAMS>`_
 
 .. _psycopg2_execution_options:
 
@@ -84,13 +92,13 @@ The following DBAPI-specific options are respected when used with
 :meth:`.Connection.execution_options`, :meth:`.Executable.execution_options`,
 :meth:`.Query.execution_options`, in addition to those not specific to DBAPIs:
 
-* ``isolation_level`` - Set the transaction isolation level for the lifespan of a
-  :class:`.Connection` (can only be set on a connection, not a statement
+* ``isolation_level`` - Set the transaction isolation level for the lifespan
+  of a :class:`.Connection` (can only be set on a connection, not a statement
   or query).   See :ref:`psycopg2_isolation_level`.
 
-* ``stream_results`` - Enable or disable usage of psycopg2 server side cursors -
-  this feature makes use of "named" cursors in combination with special
-  result handling methods so that result rows are not fully buffered.
+* ``stream_results`` - Enable or disable usage of psycopg2 server side
+  cursors - this feature makes use of "named" cursors in combination with
+  special result handling methods so that result rows are not fully buffered.
   If ``None`` or not set, the ``server_side_cursors`` option of the
   :class:`.Engine` is used.
 
@@ -101,6 +109,32 @@ The following DBAPI-specific options are respected when used with
 
   .. versionadded:: 1.0.6
 
+.. _psycopg2_batch_mode:
+
+Psycopg2 Batch Mode (Fast Execution)
+------------------------------------
+
+Modern versions of psycopg2 include a feature known as
+`Fast Execution Helpers \
+<http://initd.org/psycopg/docs/extras.html#fast-execution-helpers>`_,
+which have been shown in benchmarking to improve psycopg2's executemany()
+performance with INSERTS by multiple orders of magnitude.   SQLAlchemy
+allows this extension to be used for all ``executemany()`` style calls
+invoked by an :class:`.Engine` when used with multiple parameter sets,
+by adding the ``use_batch_mode`` flag to :func:`.create_engine`::
+
+    engine = create_engine(
+        "postgresql+psycopg2://scott:tiger@host/dbname",
+        use_batch_mode=True)
+
+Batch mode is considered to be **experimental** at this time, however may
+be enabled by default in a future release.
+
+
+.. versionadded:: 1.2.0
+
+
+
 .. _psycopg2_unicode:
 
 Unicode with Psycopg2
@@ -142,10 +176,9 @@ now supported by libpq directly.   This is enabled when ``client_encoding``
 is passed directly to ``psycopg2.connect()``, and from SQLAlchemy is passed
 using the :paramref:`.create_engine.connect_args` parameter::
 
-    # libpq direct parameter setting;
-    # only works for PostgreSQL **9.1 and above**
-    engine = create_engine("postgresql://user:pass@host/dbname",
-                           connect_args={'client_encoding': 'utf8'})
+    engine = create_engine(
+        "postgresql://user:pass@host/dbname",
+        connect_args={'client_encoding': 'utf8'})
 
     # using the query string is equivalent
     engine = create_engine("postgresql://user:pass@host/dbname?client_encoding=utf8")
@@ -260,13 +293,13 @@ The psycopg2 dialect supports these constants for isolation level:
 NOTICE logging
 ---------------
 
-The psycopg2 dialect will log PostgreSQL NOTICE messages via the
-``sqlalchemy.dialects.postgresql`` logger::
+The psycopg2 dialect will log PostgreSQL NOTICE messages
+via the ``sqlalchemy.dialects.postgresql`` logger::
 
     import logging
     logging.getLogger('sqlalchemy.dialects.postgresql').setLevel(logging.INFO)
 
-.. _psycopg2_hstore::
+.. _psycopg2_hstore:
 
 HSTORE type
 ------------
@@ -307,32 +340,39 @@ string format, on both the parameter side and the result side, will take
 place within SQLAlchemy's own marshalling logic, and not that of ``psycopg2``
 which may be more performant.
 
-"""
+"""  # noqa
 from __future__ import absolute_import
 
-import re
-import logging
-
-from ... import util, exc
 import decimal
-from ... import processors
-from ...engine import result as _result
-from ...sql import expression
-from ... import types as sqltypes
-from .base import PGDialect, PGCompiler, \
-    PGIdentifierPreparer, PGExecutionContext, \
-    ENUM, _DECIMAL_TYPES, _FLOAT_TYPES,\
-    _INT_TYPES, UUID
+import logging
+import re
+
+from .base import _DECIMAL_TYPES
+from .base import _FLOAT_TYPES
+from .base import _INT_TYPES
+from .base import ENUM
+from .base import PGCompiler
+from .base import PGDialect
+from .base import PGExecutionContext
+from .base import PGIdentifierPreparer
+from .base import UUID
 from .hstore import HSTORE
-from .json import JSON, JSONB
+from .json import JSON
+from .json import JSONB
+from ... import exc
+from ... import processors
+from ... import types as sqltypes
+from ... import util
+from ...engine import result as _result
+
 
 try:
-    from uuid import UUID as _python_UUID
+    from uuid import UUID as _python_UUID  # noqa
 except ImportError:
     _python_UUID = None
 
 
-logger = logging.getLogger('sqlalchemy.dialects.postgresql')
+logger = logging.getLogger("sqlalchemy.dialects.postgresql")
 
 
 class _PGNumeric(sqltypes.Numeric):
@@ -343,14 +383,15 @@ class _PGNumeric(sqltypes.Numeric):
         if self.asdecimal:
             if coltype in _FLOAT_TYPES:
                 return processors.to_decimal_processor_factory(
-                    decimal.Decimal,
-                    self._effective_decimal_return_scale)
+                    decimal.Decimal, self._effective_decimal_return_scale
+                )
             elif coltype in _DECIMAL_TYPES or coltype in _INT_TYPES:
                 # pg8000 returns Decimal natively for 1700
                 return None
             else:
                 raise exc.InvalidRequestError(
-                    "Unknown PG numeric type: %d" % coltype)
+                    "Unknown PG numeric type: %d" % coltype
+                )
         else:
             if coltype in _FLOAT_TYPES:
                 # pg8000 returns float natively for 701
@@ -359,12 +400,13 @@ class _PGNumeric(sqltypes.Numeric):
                 return processors.to_float
             else:
                 raise exc.InvalidRequestError(
-                    "Unknown PG numeric type: %d" % coltype)
+                    "Unknown PG numeric type: %d" % coltype
+                )
 
 
 class _PGEnum(ENUM):
     def result_processor(self, dialect, coltype):
-        if self.native_enum and util.py2k and self.convert_unicode is True:
+        if util.py2k and self.convert_unicode is True:
             # we can't easily use PG's extensions here because
             # the OID is on the fly, and we need to give it a python
             # function anyway - not really worth it.
@@ -387,7 +429,6 @@ class _PGHStore(HSTORE):
 
 
 class _PGJSON(JSON):
-
     def result_processor(self, dialect, coltype):
         if dialect._has_native_json:
             return None
@@ -396,7 +437,6 @@ class _PGJSON(JSON):
 
 
 class _PGJSONB(JSONB):
-
     def result_processor(self, dialect, coltype):
         if dialect._has_native_jsonb:
             return None
@@ -407,20 +447,22 @@ class _PGJSONB(JSONB):
 class _PGUUID(UUID):
     def bind_processor(self, dialect):
         if not self.as_uuid and dialect.use_native_uuid:
-            nonetype = type(None)
 
             def process(value):
                 if value is not None:
                     value = _python_UUID(value)
                 return value
+
             return process
 
     def result_processor(self, dialect, coltype):
         if not self.as_uuid and dialect.use_native_uuid:
+
             def process(value):
                 if value is not None:
                     value = str(value)
                 return value
+
             return process
 
 
@@ -431,8 +473,7 @@ class PGExecutionContext_psycopg2(PGExecutionContext):
     def create_server_side_cursor(self):
         # use server-side cursors:
         # http://lists.initd.org/pipermail/psycopg/2007-January/005251.html
-        ident = "c_%s_%s" % (hex(id(self))[2:],
-                             hex(_server_side_id())[2:])
+        ident = "c_%s_%s" % (hex(id(self))[2:], hex(_server_side_id())[2:])
         return self._dbapi_connection.cursor(ident)
 
     def get_result_proxy(self):
@@ -455,28 +496,21 @@ class PGExecutionContext_psycopg2(PGExecutionContext):
 
 
 class PGCompiler_psycopg2(PGCompiler):
-    def visit_mod_binary(self, binary, operator, **kw):
-        return self.process(binary.left, **kw) + " %% " + \
-            self.process(binary.right, **kw)
-
-    def post_process_text(self, text):
-        return text.replace('%', '%%')
+    pass
 
 
 class PGIdentifierPreparer_psycopg2(PGIdentifierPreparer):
-    def _escape_identifier(self, value):
-        value = value.replace(self.escape_quote, self.escape_to_quote)
-        return value.replace('%', '%%')
+    pass
 
 
 class PGDialect_psycopg2(PGDialect):
-    driver = 'psycopg2'
+    driver = "psycopg2"
     if util.py2k:
         supports_unicode_statements = False
 
     supports_server_side_cursors = True
 
-    default_paramstyle = 'pyformat'
+    default_paramstyle = "pyformat"
     # set to true based on psycopg2 version
     supports_sane_multi_rowcount = False
     execution_ctx_cls = PGExecutionContext_psycopg2
@@ -489,16 +523,16 @@ class PGDialect_psycopg2(PGDialect):
         native_jsonb=(2, 5, 4),
         sane_multi_rowcount=(2, 0, 9),
         array_oid=(2, 4, 3),
-        hstore_adapter=(2, 4)
+        hstore_adapter=(2, 4),
     )
 
     _has_native_hstore = False
     _has_native_json = False
     _has_native_jsonb = False
 
-    engine_config_types = PGDialect.engine_config_types.union([
-        ('use_native_unicode', util.asbool),
-    ])
+    engine_config_types = PGDialect.engine_config_types.union(
+        [("use_native_unicode", util.asbool)]
+    )
 
     colspecs = util.update_copy(
         PGDialect.colspecs,
@@ -510,14 +544,20 @@ class PGDialect_psycopg2(PGDialect):
             JSON: _PGJSON,
             sqltypes.JSON: _PGJSON,
             JSONB: _PGJSONB,
-            UUID: _PGUUID
-        }
+            UUID: _PGUUID,
+        },
     )
 
-    def __init__(self, server_side_cursors=False, use_native_unicode=True,
-                 client_encoding=None,
-                 use_native_hstore=True, use_native_uuid=True,
-                 **kwargs):
+    def __init__(
+        self,
+        server_side_cursors=False,
+        use_native_unicode=True,
+        client_encoding=None,
+        use_native_hstore=True,
+        use_native_uuid=True,
+        use_batch_mode=False,
+        **kwargs
+    ):
         PGDialect.__init__(self, **kwargs)
         self.server_side_cursors = server_side_cursors
         self.use_native_unicode = use_native_unicode
@@ -525,64 +565,71 @@ class PGDialect_psycopg2(PGDialect):
         self.use_native_uuid = use_native_uuid
         self.supports_unicode_binds = use_native_unicode
         self.client_encoding = client_encoding
-        if self.dbapi and hasattr(self.dbapi, '__version__'):
-            m = re.match(r'(\d+)\.(\d+)(?:\.(\d+))?',
-                         self.dbapi.__version__)
+        self.psycopg2_batch_mode = use_batch_mode
+        if self.dbapi and hasattr(self.dbapi, "__version__"):
+            m = re.match(r"(\d+)\.(\d+)(?:\.(\d+))?", self.dbapi.__version__)
             if m:
                 self.psycopg2_version = tuple(
-                    int(x)
-                    for x in m.group(1, 2, 3)
-                    if x is not None)
+                    int(x) for x in m.group(1, 2, 3) if x is not None
+                )
 
     def initialize(self, connection):
         super(PGDialect_psycopg2, self).initialize(connection)
-        self._has_native_hstore = self.use_native_hstore and \
-            self._hstore_oids(connection.connection) \
-            is not None
-        self._has_native_json = \
-            self.psycopg2_version >= self.FEATURE_VERSION_MAP['native_json']
-        self._has_native_jsonb = \
-            self.psycopg2_version >= self.FEATURE_VERSION_MAP['native_jsonb']
+        self._has_native_hstore = (
+            self.use_native_hstore
+            and self._hstore_oids(connection.connection) is not None
+        )
+        self._has_native_json = (
+            self.psycopg2_version >= self.FEATURE_VERSION_MAP["native_json"]
+        )
+        self._has_native_jsonb = (
+            self.psycopg2_version >= self.FEATURE_VERSION_MAP["native_jsonb"]
+        )
 
         # http://initd.org/psycopg/docs/news.html#what-s-new-in-psycopg-2-0-9
-        self.supports_sane_multi_rowcount = \
-            self.psycopg2_version >= \
-            self.FEATURE_VERSION_MAP['sane_multi_rowcount']
+        self.supports_sane_multi_rowcount = (
+            self.psycopg2_version
+            >= self.FEATURE_VERSION_MAP["sane_multi_rowcount"]
+            and not self.psycopg2_batch_mode
+        )
 
     @classmethod
     def dbapi(cls):
         import psycopg2
+
         return psycopg2
 
     @classmethod
     def _psycopg2_extensions(cls):
         from psycopg2 import extensions
+
         return extensions
 
     @classmethod
     def _psycopg2_extras(cls):
         from psycopg2 import extras
+
         return extras
 
     @util.memoized_property
     def _isolation_lookup(self):
         extensions = self._psycopg2_extensions()
         return {
-            'AUTOCOMMIT': extensions.ISOLATION_LEVEL_AUTOCOMMIT,
-            'READ COMMITTED': extensions.ISOLATION_LEVEL_READ_COMMITTED,
-            'READ UNCOMMITTED': extensions.ISOLATION_LEVEL_READ_UNCOMMITTED,
-            'REPEATABLE READ': extensions.ISOLATION_LEVEL_REPEATABLE_READ,
-            'SERIALIZABLE': extensions.ISOLATION_LEVEL_SERIALIZABLE
+            "AUTOCOMMIT": extensions.ISOLATION_LEVEL_AUTOCOMMIT,
+            "READ COMMITTED": extensions.ISOLATION_LEVEL_READ_COMMITTED,
+            "READ UNCOMMITTED": extensions.ISOLATION_LEVEL_READ_UNCOMMITTED,
+            "REPEATABLE READ": extensions.ISOLATION_LEVEL_REPEATABLE_READ,
+            "SERIALIZABLE": extensions.ISOLATION_LEVEL_SERIALIZABLE,
         }
 
     def set_isolation_level(self, connection, level):
         try:
-            level = self._isolation_lookup[level.replace('_', ' ')]
+            level = self._isolation_lookup[level.replace("_", " ")]
         except KeyError:
             raise exc.ArgumentError(
                 "Invalid value '%s' for isolation_level. "
-                "Valid isolation levels for %s are %s" %
-                (level, self.name, ", ".join(self._isolation_lookup))
+                "Valid isolation levels for %s are %s"
+                % (level, self.name, ", ".join(self._isolation_lookup))
             )
 
         connection.set_isolation_level(level)
@@ -593,61 +640,86 @@ class PGDialect_psycopg2(PGDialect):
 
         fns = []
         if self.client_encoding is not None:
+
             def on_connect(conn):
                 conn.set_client_encoding(self.client_encoding)
+
             fns.append(on_connect)
 
         if self.isolation_level is not None:
+
             def on_connect(conn):
                 self.set_isolation_level(conn, self.isolation_level)
+
             fns.append(on_connect)
 
         if self.dbapi and self.use_native_uuid:
+
             def on_connect(conn):
                 extras.register_uuid(None, conn)
+
             fns.append(on_connect)
 
         if self.dbapi and self.use_native_unicode:
+
             def on_connect(conn):
                 extensions.register_type(extensions.UNICODE, conn)
                 extensions.register_type(extensions.UNICODEARRAY, conn)
+
             fns.append(on_connect)
 
         if self.dbapi and self.use_native_hstore:
+
             def on_connect(conn):
                 hstore_oids = self._hstore_oids(conn)
                 if hstore_oids is not None:
                     oid, array_oid = hstore_oids
-                    kw = {'oid': oid}
+                    kw = {"oid": oid}
                     if util.py2k:
-                        kw['unicode'] = True
-                    if self.psycopg2_version >= \
-                            self.FEATURE_VERSION_MAP['array_oid']:
-                        kw['array_oid'] = array_oid
+                        kw["unicode"] = True
+                    if (
+                        self.psycopg2_version
+                        >= self.FEATURE_VERSION_MAP["array_oid"]
+                    ):
+                        kw["array_oid"] = array_oid
                     extras.register_hstore(conn, **kw)
+
             fns.append(on_connect)
 
         if self.dbapi and self._json_deserializer:
+
             def on_connect(conn):
                 if self._has_native_json:
                     extras.register_default_json(
-                        conn, loads=self._json_deserializer)
+                        conn, loads=self._json_deserializer
+                    )
                 if self._has_native_jsonb:
                     extras.register_default_jsonb(
-                        conn, loads=self._json_deserializer)
+                        conn, loads=self._json_deserializer
+                    )
+
             fns.append(on_connect)
 
         if fns:
+
             def on_connect(conn):
                 for fn in fns:
                     fn(conn)
+
             return on_connect
         else:
             return None
 
+    def do_executemany(self, cursor, statement, parameters, context=None):
+        if self.psycopg2_batch_mode:
+            extras = self._psycopg2_extras()
+            extras.execute_batch(cursor, statement, parameters)
+        else:
+            cursor.executemany(statement, parameters)
+
     @util.memoized_instancemethod
     def _hstore_oids(self, conn):
-        if self.psycopg2_version >= self.FEATURE_VERSION_MAP['hstore_adapter']:
+        if self.psycopg2_version >= self.FEATURE_VERSION_MAP["hstore_adapter"]:
             extras = self._psycopg2_extras()
             oids = extras.HstoreAdapter.get_oids(conn)
             if oids is not None and oids[0]:
@@ -655,9 +727,9 @@ class PGDialect_psycopg2(PGDialect):
         return None
 
     def create_connect_args(self, url):
-        opts = url.translate_connect_args(username='user')
-        if 'port' in opts:
-            opts['port'] = int(opts['port'])
+        opts = url.translate_connect_args(username="user")
+        if "port" in opts:
+            opts["port"] = int(opts["port"])
         opts.update(url.query)
         return ([], opts)
 
@@ -667,7 +739,7 @@ class PGDialect_psycopg2(PGDialect):
             # present on old psycopg2 versions.   Also,
             # this flag doesn't actually help in a lot of disconnect
             # situations, so don't rely on it.
-            if getattr(connection, 'closed', False):
+            if getattr(connection, "closed", False):
                 return True
 
             # checks based on strings.  in the case that .closed
@@ -676,27 +748,29 @@ class PGDialect_psycopg2(PGDialect):
             for msg in [
                 # these error messages from libpq: interfaces/libpq/fe-misc.c
                 # and interfaces/libpq/fe-secure.c.
-                'terminating connection',
-                'closed the connection',
-                'connection not open',
-                'could not receive data from server',
-                'could not send data to server',
+                "terminating connection",
+                "closed the connection",
+                "connection not open",
+                "could not receive data from server",
+                "could not send data to server",
                 # psycopg2 client errors, psycopg2/conenction.h,
                 # psycopg2/cursor.h
-                'connection already closed',
-                'cursor already closed',
+                "connection already closed",
+                "cursor already closed",
                 # not sure where this path is originally from, it may
                 # be obsolete.   It really says "losed", not "closed".
-                'losed the connection unexpectedly',
+                "losed the connection unexpectedly",
                 # these can occur in newer SSL
-                'connection has been closed unexpectedly',
-                'SSL SYSCALL error: Bad file descriptor',
-                'SSL SYSCALL error: EOF detected',
-                'SSL error: decryption failed or bad record mac',
+                "connection has been closed unexpectedly",
+                "SSL SYSCALL error: Bad file descriptor",
+                "SSL SYSCALL error: EOF detected",
+                "SSL error: decryption failed or bad record mac",
+                "SSL SYSCALL error: Operation timed out",
             ]:
                 idx = str_e.find(msg)
                 if idx >= 0 and '"' not in str_e[:idx]:
                     return True
         return False
 
+
 dialect = PGDialect_psycopg2
